// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
import { getTraceParentHeader, createSpanFunction, SpanKind, SpanStatusCode, isSpanContextValid } from "@azure/core-tracing";
import { BaseRequestPolicy } from "./requestPolicy";
import { URLBuilder } from "../url";
const createSpan = createSpanFunction({
    packagePrefix: "",
    namespace: ""
});
export function tracingPolicy(tracingOptions = {}) {
    return {
        create(nextPolicy, options) {
            return new TracingPolicy(nextPolicy, options, tracingOptions);
        }
    };
}
export class TracingPolicy extends BaseRequestPolicy {
    constructor(nextPolicy, options, tracingOptions) {
        super(nextPolicy, options);
        this.userAgent = tracingOptions.userAgent;
    }
    async sendRequest(request) {
        if (!request.tracingContext) {
            return this._nextPolicy.sendRequest(request);
        }
        // create a new span
        const path = URLBuilder.parse(request.url).getPath() || "/";
        const { span } = createSpan(path, {
            tracingOptions: {
                spanOptions: Object.assign(Object.assign({}, request.spanOptions), { kind: SpanKind.CLIENT }),
                tracingContext: request.tracingContext
            }
        });
        span.setAttributes({
            "http.method": request.method,
            "http.url": request.url,
            requestId: request.requestId
        });
        if (this.userAgent) {
            span.setAttribute("http.user_agent", this.userAgent);
        }
        try {
            // set headers
            const spanContext = span.spanContext();
            const traceParentHeader = getTraceParentHeader(spanContext);
            if (traceParentHeader && isSpanContextValid(spanContext)) {
                request.headers.set("traceparent", traceParentHeader);
                const traceState = spanContext.traceState && spanContext.traceState.serialize();
                // if tracestate is set, traceparent MUST be set, so only set tracestate after traceparent
                if (traceState) {
                    request.headers.set("tracestate", traceState);
                }
            }
            const response = await this._nextPolicy.sendRequest(request);
            span.setAttribute("http.status_code", response.status);
            const serviceRequestId = response.headers.get("x-ms-request-id");
            if (serviceRequestId) {
                span.setAttribute("serviceRequestId", serviceRequestId);
            }
            span.setStatus({
                code: SpanStatusCode.OK
            });
            return response;
        }
        catch (err) {
            span.setStatus({
                code: SpanStatusCode.ERROR,
                message: err.message
            });
            span.setAttribute("http.status_code", err.statusCode);
            throw err;
        }
        finally {
            span.end();
        }
    }
}
//# sourceMappingURL=tracingPolicy.js.map